"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildApp = void 0;
const lowcode_builder_1 = require("@cloudbase/lowcode-builder");
const fs_extra_1 = __importDefault(require("fs-extra"));
const chalk_1 = __importDefault(require("chalk"));
const common_1 = require("../../common");
const path_1 = __importDefault(require("path"));
const DIST_PATH = './dist';
const DEFAULT_INPUTS = {
    config: 'input.json',
    debug: false,
    runtime: process.env.CLOUDBASE_CIID ? lowcode_builder_1.RUNTIME.CI : lowcode_builder_1.RUNTIME.NONE,
    appId: 'test',
    buildTypeList: [lowcode_builder_1.BuildType.WEB],
    generateMpType: 'app',
    subAppSerializeDataList: [],
    dependencies: [],
    extraData: { isComposite: false, compProps: {} },
    mpAppId: '',
    deployOptions: {
        mode: lowcode_builder_1.DEPLOY_MODE.PREVIEW,
        publicPath: '',
    },
    ignoreInstall: false,
    clean: false,
    out: DIST_PATH,
};
class Builder {
    constructor(api, inputs) {
        this.api = api;
        this.logger = api.logger || common_1.Logger;
        this.inputJSONPath = path_1.default.resolve(api.projectPath, inputs.config || DEFAULT_INPUTS.config);
        let params = fs_extra_1.default.existsSync(this.inputJSONPath) ? fs_extra_1.default.readJsonSync(this.inputJSONPath) : {};
        this.resolvedInputs = this.normalizInputs(api, resolveInputs(inputs, resolveInputs(params, DEFAULT_INPUTS)));
        if (fs_extra_1.default.existsSync(this.inputJSONPath)) {
            fs_extra_1.default.writeFileSync(this.inputJSONPath, JSON.stringify(this.resolvedInputs, undefined, 2));
        }
    }
    normalizInputs(api, inputs) {
        if (!inputs.mainAppSerializeData) {
            throw new Error('缺少必须参数: mainAppSerializeData');
        }
        // 副作用方法，修改原值
        return (0, lowcode_builder_1.normalizeInputs)(inputs, { envId: api.envId });
    }
    async build(data = undefined) {
        var _a;
        this.logger.info('👉 开始生成文件...');
        const outputPath = path_1.default.resolve(this.api.projectPath, this.resolvedInputs.out);
        if (this.resolvedInputs.clean) {
            this.logger.info(`👉 清空输出目录: ${outputPath}`);
            fs_extra_1.default.emptyDirSync(outputPath);
        }
        if (!data) {
            let { appId, mpAppId, mainAppSerializeData, subAppSerializeDataList, dependencies, runtime, buildTypeList, generateMpType, deployOptions, extraData = { isComposite: false, compProps: {} }, ignoreInstall, cdnEndpoints, } = this.resolvedInputs;
            data = {
                cals: mainAppSerializeData,
                subAppCalsList: subAppSerializeDataList,
                dependencies,
                appKey: appId,
                runtime,
                ignoreInstall,
                buildTypeList,
                mode: (deployOptions === null || deployOptions === void 0 ? void 0 : deployOptions.mode) === lowcode_builder_1.DEPLOY_MODE.UPLOAD ? lowcode_builder_1.WebpackModeType.PRODUCTION : lowcode_builder_1.WebpackModeType.NONE,
                deployOptions,
                generateMpType,
                // isCleanDistDir: false,
                plugins: [],
                extraData,
                resourceAppId: mpAppId,
                domain: (_a = mainAppSerializeData.extra) === null || _a === void 0 ? void 0 : _a.domain,
                output: { path: outputPath },
                cdnEndpoints: cdnEndpoints,
                isPrivateMode: !!this.api.privateSettings,
            };
        }
        const appPath = await (0, lowcode_builder_1.buildWedaApp)(data);
        this.logger.info(`👉 生成到目录成功：${chalk_1.default.bold.yellow(appPath)}`);
        return { output: outputPath, app: appPath };
    }
}
function resolveInputs(inputs, defaultInputs) {
    return Object.assign({}, defaultInputs, inputs);
}
async function buildApp(ctx, params) {
    var _a, _b, _c;
    const builder = new Builder(ctx, params);
    try {
        const { output, app: appDir } = await builder.build();
        let inputs;
        if (builder.inputJSONPath && fs_extra_1.default.existsSync(builder.inputJSONPath)) {
            inputs = fs_extra_1.default.readJsonSync(builder.inputJSONPath);
        }
        const previewPath = (0, lowcode_builder_1.buildAsWebByBuildType)(builder.resolvedInputs.buildTypeList)
            ? path_1.default.join(appDir, 'preview')
            : appDir;
        let cloudbaseRc = {
            version: '2.0',
            envId: '{{env.ENV_ID}}',
            lowcode: {
                'publish:app': {
                    inputs: {
                        appId: builder.resolvedInputs.appId,
                        buildTypeList: builder.resolvedInputs.buildTypeList,
                        projectPath: previewPath,
                        mpAppId: builder.resolvedInputs.mpAppId || undefined,
                        credential: (inputs === null || inputs === void 0 ? void 0 : inputs.credential) || undefined,
                        storage: inputs === null || inputs === void 0 ? void 0 : inputs.storage,
                        webOptions: inputs === null || inputs === void 0 ? void 0 : inputs.webOptions,
                        deployOptions: {
                            mpDeployPrivateKey: inputs.mpDeployPrivateKey || undefined,
                            ...builder.resolvedInputs.deployOptions,
                        },
                    },
                },
            },
        };
        const currentCloudRcPath = path_1.default.join(ctx.projectPath, 'cloudbaserc.json');
        const currentEnvPath = path_1.default.join(ctx.projectPath, '.env');
        if (fs_extra_1.default.existsSync(currentCloudRcPath)) {
            const current = await fs_extra_1.default.readJSON(currentCloudRcPath);
            cloudbaseRc = {
                ...cloudbaseRc,
                ...current,
                lowcode: {
                    ...current.lowcode,
                    'publish:app': {
                        ...(((_a = current === null || current === void 0 ? void 0 : current.lowcode) === null || _a === void 0 ? void 0 : _a['publish:app']) || {}),
                        inputs: {
                            ...cloudbaseRc.lowcode['publish:app'].inputs,
                            ...(((_c = (_b = current === null || current === void 0 ? void 0 : current.lowcode) === null || _b === void 0 ? void 0 : _b['publish:app']) === null || _c === void 0 ? void 0 : _c.inputs) || {}),
                        },
                    },
                },
            };
        }
        await fs_extra_1.default.writeFile(path_1.default.join(output, 'cloudbaserc.json'), JSON.stringify(cloudbaseRc, undefined, 2));
        if (fs_extra_1.default.existsSync(currentEnvPath)) {
            fs_extra_1.default.copyFileSync(currentEnvPath, path_1.default.join(output, '.env'));
        }
        return { src: appDir, preivew: previewPath };
    }
    catch (e) {
        console.error(e);
        process.exit(-1);
    }
}
exports.buildApp = buildApp;
