"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deployApp = void 0;
const lowcode_deployer_1 = require("@cloudbase/lowcode-deployer");
const lowcode_builder_1 = require("@cloudbase/lowcode-builder");
const fs_extra_1 = __importDefault(require("fs-extra"));
const common_1 = require("../../common");
const path_1 = __importDefault(require("path"));
const MINIPROGRAM_SETTING = {
    es6: true,
    es7: true,
    minify: true,
    codeProtect: false,
    useCompilerPlugins: ['typescript'],
};
const DEFAULT_INPUTS = {
    _inputFile: 'input.json',
    appId: 'test',
    mpAppId: '',
    buildTypeList: [lowcode_builder_1.BuildType.WEB],
    deployOptions: {
        mode: (process.env.deployMode || lowcode_builder_1.DEPLOY_MODE.PREVIEW),
    },
};
class Deoloyer {
    constructor(api, inputs) {
        var _a, _b;
        this.api = api;
        this.logger = api.logger || common_1.Logger;
        let inputJSONPath = path_1.default.resolve(api.projectPath, inputs._inputFile || DEFAULT_INPUTS._inputFile);
        let params = fs_extra_1.default.existsSync(inputJSONPath) ? fs_extra_1.default.readJsonSync(inputJSONPath) : {};
        this._resolvedInputs = this.normalizInputs(api, resolveInputs(inputs, resolveInputs(params, DEFAULT_INPUTS)));
        const { appId, mpAppId, envId, buildTypeList, deployOptions, projectPath, webOptions = {} } = this._resolvedInputs;
        const type = (0, lowcode_builder_1.buildAsWebByBuildType)(buildTypeList) ? 'WEB' : 'MP';
        const mode = deployOptions.mode === lowcode_builder_1.DEPLOY_MODE.UPLOAD ? lowcode_deployer_1.EDeployMode.PRODUCTION : lowcode_deployer_1.EDeployMode.PREVIEW;
        const deployerOptions = {
            logger: this.logger,
            type,
            mode,
            appId,
            envId,
            projectPath,
            ignores: [],
            mpAppId,
            credential: this._resolvedInputs.credential || this.api.privateSettings.credential,
            artifactOptions: process.env.CLOUDBASE_CIID
                ? {
                    ...this._resolvedInputs.storage,
                    baseKey: `lca/${appId}${process.env.CLOUDBASE_CIID ? `/${process.env.CLOUDBASE_CIID}` : ''}`,
                }
                : undefined,
        };
        if (type === 'MP') {
            let projectJson = fs_extra_1.default.readJsonSync(path_1.default.resolve(projectPath, 'project.config.json'));
            let { cloudfunctionRoot, miniprogramRoot = './' } = projectJson;
            deployerOptions.mode =
                type === 'MP' && deployOptions.mpAppId !== deployOptions.targetMpAppId
                    ? lowcode_deployer_1.EDeployMode.PRODUCTION
                    : deployerOptions.mode;
            deployerOptions.ignores = ['node_modules/**/*'].concat(cloudfunctionRoot ? [path_1.default.join(cloudfunctionRoot, '**/*')] : []);
            deployerOptions.miniprogramOptions = {
                privateKeyPath: path_1.default.join(projectPath, `./private.${deployOptions.mpAppId}.key`),
                uploadOptions: {
                    version: (deployOptions === null || deployOptions === void 0 ? void 0 : deployOptions.version) || '1.0.0',
                    desc: (deployOptions === null || deployOptions === void 0 ? void 0 : deployOptions.description) || '',
                    setting: MINIPROGRAM_SETTING,
                },
                previewOptions: {
                    qrcodeOutputPath: path_1.default.resolve(projectPath, './qrcode.jpg'),
                    pagePath: (_b = (_a = fs_extra_1.default.readJsonSync(path_1.default.resolve(projectPath, miniprogramRoot, 'app.json'))) === null || _a === void 0 ? void 0 : _a.pages) === null || _b === void 0 ? void 0 : _b[0],
                    setting: MINIPROGRAM_SETTING,
                },
            };
        }
        else {
            deployerOptions.webOptions = {
                buildTypeList,
                ...webOptions,
            };
        }
        this.instance = new lowcode_deployer_1.deployer(deployerOptions);
    }
    normalizInputs(api, inputs) {
        var _a, _b, _c, _d;
        /**
         * mainAppSerializeData  为了兼容 lowcode plugin 参数
         */
        inputs.envId = ((_b = (_a = inputs.mainAppSerializeData) === null || _a === void 0 ? void 0 : _a.extra) === null || _b === void 0 ? void 0 : _b.envid) || inputs.envId || api.envId;
        inputs.projectPath = inputs.projectPath || api.projectPath;
        inputs.webOptions = inputs.webOptions || {};
        if (!(0, lowcode_builder_1.buildAsWebByBuildType)(inputs.buildTypeList)) {
            if (!((_c = inputs.deployOptions) === null || _c === void 0 ? void 0 : _c.mpDeployPrivateKey)) {
                if (inputs.mpDeployPrivateKey) {
                    inputs.deployOptions.mpDeployPrivateKey = inputs.mpDeployPrivateKey;
                }
                else if (process.env.mpDeployPrivateKey) {
                    inputs.deployOptions.mpDeployPrivateKey = process.env.mpDeployPrivateKey;
                }
            }
            if (!((_d = inputs.deployOptions) === null || _d === void 0 ? void 0 : _d.mpAppId)) {
                inputs.deployOptions.mpAppId = inputs.mpAppId;
            }
            if (inputs.deployOptions.mpDeployPrivateKey) {
                this._generateMPDeployPrivateKey = true;
                fs_extra_1.default.writeFileSync(path_1.default.join(inputs.projectPath, `./private.${inputs.deployOptions.mpAppId}.key`), inputs.deployOptions.mpDeployPrivateKey, 'base64');
            }
        }
        return inputs;
    }
    async deploy() {
        try {
            this.logger.info('👉 开始部署产物...');
            const res = await this.instance.deploy();
            this.logger.info('👉 部署成功');
            return res;
        }
        catch (e) {
            throw e;
        }
        finally {
            fs_extra_1.default.removeSync(path_1.default.join(this._resolvedInputs.projectPath, `./private.${this._resolvedInputs.deployOptions.mpAppId}.key`));
        }
    }
}
function resolveInputs(inputs, defaultInputs) {
    return Object.assign({}, defaultInputs, inputs);
}
async function deployApp(ctx, params) {
    try {
        const instance = new Deoloyer(ctx, params);
        const res = await instance.deploy();
        return res;
    }
    catch (e) {
        console.error(e);
        process.exit(-1);
    }
}
exports.deployApp = deployApp;
