"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.bindRouter = exports.PreviewService = void 0;
const common_1 = require("../../common");
const child_process_1 = require("child_process");
const util_1 = require("util");
const path_1 = __importDefault(require("path"));
const os_1 = __importDefault(require("os"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const toolbox_1 = require("@cloudbase/toolbox");
const constant_1 = require("./constant");
const lodash_1 = __importDefault(require("lodash"));
const lowcode_builder_1 = require("@cloudbase/lowcode-builder");
const common_2 = require("@cloudbase/lowcode-builder/lib/builder/types/common");
const types_1 = require("@cloudbase/lowcode-builder/lib/types");
const chalk_1 = __importDefault(require("chalk"));
const semver_1 = __importDefault(require("semver"));
class PreviewService {
    constructor(info) {
        this.localCompsMap = {};
        this.info = info;
        this.root = path_1.default.resolve(os_1.default.homedir(), 'cloudbase-lowcode/preview');
    }
    async start(appid) {
        (0, common_1.debug)('preview check login state start');
        // 探测是否已登录
        try {
            const res = await this.runIDECommand([...['cloud', 'env', 'list'], ...(!!appid ? ['--appid', appid] : [])]);
            if (res.stdout.match(constant_1.WxmpError.NO_LOGIN) || res.stderr.match(constant_1.WxmpError.NO_LOGIN)) {
                throw new toolbox_1.CloudBaseError(`微信小程序IDE错误: 请打开微信小程序IDE并扫码登录！`);
            }
            else if (res.stdout.match(constant_1.WxmpError.INVALID_APPID) || res.stderr.match(constant_1.WxmpError.INVALID_APPID)) {
                throw new toolbox_1.CloudBaseError(`微信小程序IDE错误: 当前登录态不属于[${appid}]，请重新登录！`);
            }
        }
        catch (e) {
            if (e instanceof toolbox_1.CloudBaseError) {
                throw e;
            }
            else {
                throw new toolbox_1.CloudBaseError(`微信IDE调用错误: ${e}`);
            }
        }
        this.lastAppid = appid;
        (0, common_1.debug)('preview check login state end');
    }
    // 设置本地组件库路径
    async setLocalCompsPath(name, localCompsPath, debugAssetsPath) {
        this.localCompsMap[name] = { localCompsPath, debugAssetsPath };
    }
    async runIDECommand(options, projectPath) {
        const fixedOptions = [...options, ...['--lang', 'zh'], ...(projectPath ? ['--project', projectPath] : [])];
        const cwd = this.info.wxDevtoolPath || (0, constant_1.getDefaultWxCliPath)();
        const cmd = `echo y | ${(0, constant_1.getWxCliCmd)()} ${fixedOptions.join(' ')}`;
        (0, common_1.debug)(`preview runIDECommand cmd: ${cmd}`);
        (0, common_1.debug)(`preview runIDECommand cwd: ${cwd}`);
        if (!fs_extra_1.default.pathExistsSync(cwd)) {
            const lowcodeCmd = this.info.mode === 'debug' ? 'debug' : 'watch';
            throw new toolbox_1.CloudBaseError(`未找到微信开发者工具的安装路径, 请通过【tcb lowcode ${lowcodeCmd} --wx-devtool-path="your-wx-dev-tool-path"】指定微信开发者工具的安装路径并开启监听`);
        }
        const res = await (0, util_1.promisify)(child_process_1.exec)(cmd, {
            env: process.env,
            cwd,
        });
        return res;
    }
    async runShell(command, options = {}) {
        const res = await (0, util_1.promisify)(child_process_1.exec)(command, options);
        return res;
    }
    async checkVersion(command) {
        const versionRes = await this.runShell(command);
        const version = this.getVersion(versionRes.stdout);
        common_1.Logger.info(`👉 CloudBase CLI: ${version}`);
        if (version === '') {
            common_1.Logger.error(`❗检查到没有安装@cloudbase/cli。请执行${chalk_1.default.bold.yellow('npm i @cloudbase/cli@latest -g')}进行安装`);
            throw new toolbox_1.CloudBaseError(`${JSON.stringify({ stdout: 'version_null' })}`);
        }
        if (semver_1.default.lt(version, constant_1.BASE_VERSION)) {
            common_1.Logger.error(`❗检查到@cloudbase/cli版本低于${constant_1.BASE_VERSION}。请执行${chalk_1.default.bold.yellow('npm i @cloudbase/cli@latest -g')}进行安装`);
            throw new toolbox_1.CloudBaseError(`${JSON.stringify({ stdout: 'version_too_old' })}`);
        }
    }
    getVersion(stdout = '') {
        const regex = /CloudBase CLI (.*)\n/; // 走到这里逻辑不可能没有安装cli，除非这个文案改了
        let version = '';
        if (stdout && typeof stdout === 'string' && regex.exec(stdout)) {
            const regexRes = regex.exec(stdout);
            version = lodash_1.default.get(regexRes, '1', '');
        }
        return version;
    }
    async buildApp(data) {
        var _a, _b;
        const projectPath = (_a = data.output) === null || _a === void 0 ? void 0 : _a.path;
        await (0, lowcode_builder_1.buildWedaApp)({
            ...data,
            output: { path: projectPath },
            ignoreInstall: !((_b = this.info) === null || _b === void 0 ? void 0 : _b.forceInstall),
        });
        if (!fs_extra_1.default.existsSync(path_1.default.join(projectPath, 'node_modules'))) {
            common_1.Logger.error(`❗检测到${chalk_1.default.bold.yellow(projectPath)}目录安装node_modules失败，请打开该目录执行${chalk_1.default.bold.yellow('npm i')}进行手动安装`);
            throw new toolbox_1.CloudBaseError(`${JSON.stringify({ type: 'NO_NODE_MODULES', dir: projectPath })}`);
        }
    }
    async buildMiniprogramNpm(projectPath) {
        let hasConsole = false;
        try {
            if (fs_extra_1.default.existsSync(path_1.default.join(projectPath, 'miniprogram_npm'))) {
                common_1.Logger.info(`👉 检测到${projectPath}目录已经有miniprogram_npm，跳过构建`);
                return;
            }
            const res = await this.runIDECommand(['build-npm'], projectPath);
            if (res.stdout.toLocaleLowerCase().includes('error')) {
                hasConsole = true;
                common_1.Logger.error(`❗微信开发者工具构建miniprogram_npm出错: ${JSON.stringify(res)}`);
                throw new toolbox_1.CloudBaseError(`${JSON.stringify({ ...res, dir: projectPath })}`);
            }
            common_1.Logger.info(`${chalk_1.default.bold.cyan('👉 构建miniprogram_npm成功')}`);
        }
        catch (err) {
            !hasConsole && common_1.Logger.error(`❗微信开发者工具构建miniprogram_npm出错: ${JSON.stringify(err)}`);
            throw new toolbox_1.CloudBaseError(`${JSON.stringify({ ...err, dir: projectPath })}`);
        }
    }
    async openIDE(projectPath) {
        let hasConsole = false;
        try {
            const res = await this.runIDECommand(['open'], projectPath);
            if (res.stdout.toLocaleLowerCase().includes('error')) {
                hasConsole = true;
                common_1.Logger.error(`❗打开微信开发者工具错误: ${JSON.stringify(res)}`);
                throw new toolbox_1.CloudBaseError(`${JSON.stringify({ ...res, dir: projectPath })}`);
            }
        }
        catch (err) {
            !hasConsole && common_1.Logger.error(`❗打开微信开发者工具错误: ${JSON.stringify(err)}`);
            throw new toolbox_1.CloudBaseError(`${JSON.stringify({ ...err, dir: projectPath })}`);
        }
    }
    /**
     * 接入新版lowcode-builder
     * @param data
     */
    async build(data) {
        var _a;
        common_1.Logger.info('👉 检测@cloudbase/cli版本...');
        await this.checkVersion('tcb -v');
        // 输出目录，生成到该目录的mp下
        const projectPath = (_a = data.output) === null || _a === void 0 ? void 0 : _a.path;
        // if (fse.existsSync(projectPath) && os.platform() !== 'win32') {
        //   // TODO: LCF编译后, 在WINDOWS下会持有文件符, 导致清空文件夹失败
        //   fse.emptyDirSync(projectPath);
        // }
        common_1.Logger.info('👉 开始生成文件...');
        await this.buildApp(data);
        common_1.Logger.info(`👉 生成到目录成功：${chalk_1.default.bold.yellow(projectPath)}`);
        common_1.Logger.info('👉 开始构建小程序miniprogram_npm包...');
        await this.buildMiniprogramNpm(projectPath);
        common_1.Logger.info('👉 开始打开微信开发者工具...');
        await this.openIDE(projectPath);
        common_1.Logger.info(`👉 打开微信开发者工具成功。如没有正常打开，可直接手动打开这个目录：${chalk_1.default.bold.yellow(projectPath)}`);
        const { materialsDir } = (0, lowcode_builder_1.getCompileDirs)();
        common_1.Logger.info(`👉 如遇到问题，建议删除组件缓存重试下。删除目录：${chalk_1.default.bold.yellow(materialsDir)}`);
    }
    /**
     * 组建lowcode-builder buildWedaApp入参
     * @param IBuildDataParams
     * @returns
     */
    async getBuilderData({ mpAppId, resourceAppId, content, previewType, envId, endpointType, }) {
        try {
            const contentJSON = JSON.parse(content);
            const appId = lodash_1.default.get(contentJSON, 'appId');
            const mainAppSerializeData = JSON.parse(lodash_1.default.get(contentJSON, 'mainAppSerializeData', '{}'));
            const dependencies = (await this.getDependenciesUrl(lodash_1.default.get(contentJSON, 'dependencies', []), this.info.mode, this.info.port));
            lodash_1.default.set(mainAppSerializeData, 'extra.envId', envId);
            return {
                appKey: appId,
                cals: {
                    appId,
                    ...mainAppSerializeData,
                    dataSources: lodash_1.default.get(contentJSON, 'dataSources', []),
                },
                subAppCalsList: lodash_1.default.get(contentJSON, 'subAppSerializeDataStrList', []),
                dependencies,
                buildTypeList: [common_2.BuildType.MP],
                deployOptions: {
                    mode: types_1.DEPLOY_MODE.PREVIEW,
                    mpAppId,
                    targetMpAppId: mpAppId,
                },
                domain: lodash_1.default.get(mainAppSerializeData, 'extra.domain', ''),
                resourceAppId,
                output: {
                    path: path_1.default.resolve(this.root, mpAppId || 'trustyAppid'),
                },
                endpointType,
            };
        }
        catch (err) {
            throw new toolbox_1.CloudBaseError(`tcb lowcode watch 获取数据出错：${err}`);
        }
    }
    /**
     * debug模式下需要重新处理dependiences的mpPkgUrl，并提供zip包
     * @param dependencies
     * @param mode
     * @param port
     * @returns
     */
    async getDependenciesUrl(dependencies, mode, port) {
        if (mode === 'debug') {
            await dependencies.reduce(async (promise, lib) => {
                var _a;
                await promise;
                if (lib.type === 'local') {
                    const { localCompsPath, debugAssetsPath } = this.localCompsMap[lib.name];
                    if (!localCompsPath || !fs_extra_1.default.existsSync(localCompsPath)) {
                        throw new toolbox_1.CloudBaseError(`本地组件库错误: ${localCompsPath} 不存在!`);
                    }
                    // 拷贝 src
                    fs_extra_1.default.copySync(localCompsPath, path_1.default.resolve(debugAssetsPath, lib.name, 'src'));
                    // 写入 meta.json
                    fs_extra_1.default.writeJSONSync(path_1.default.resolve(debugAssetsPath, lib.name, 'meta.json'), {
                        schemaVersion: common_1.COMPONENTS_CONFIG_SCHEMA_VERSION,
                        components: ((_a = lib.components) === null || _a === void 0 ? void 0 : _a.reduce((pre, cur) => {
                            pre[cur.name] = cur;
                            return pre;
                        }, {})) || {},
                        actions: {},
                    });
                    // 压缩打包
                    await (0, toolbox_1.zipDir)(path_1.default.resolve(debugAssetsPath, lib.name), path_1.default.resolve(debugAssetsPath, `${lib.name}.zip`), [
                        'node_modules/**',
                        '**/node_modules/**',
                        '.DS_Store',
                        '**/.DS_Store',
                    ]);
                    lib.mpPkgUrl = `http://127.0.0.1:${port}/assets/${lib.name}.zip`;
                }
            }, Promise.resolve());
        }
        return dependencies;
    }
}
exports.PreviewService = PreviewService;
function bindRouter(server, service) {
    server.addListener('/preview/build', async (ctx) => {
        // @ts-ignore
        const data = await service.getBuilderData(ctx.request.body);
        await service.build(data);
        ctx.success();
    });
}
exports.bindRouter = bindRouter;
