"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Watcher = exports.Builder = void 0;
const child_process_1 = require("child_process");
const chokidar_1 = require("chokidar");
const path_1 = require("path");
const fs_1 = require("fs");
const url_1 = require("url");
const toolbox_1 = require("@cloudbase/toolbox");
const chalk_1 = __importDefault(require("chalk"));
const cloud_api_1 = require("@cloudbase/cloud-api");
const lodash_1 = __importDefault(require("lodash"));
const DEFAULT_FRAMEWORK_COMMAND = 'cloudbase framework deploy';
const DEFAULT_RC_CONTENT = {
    version: '2.0',
    envId: '{{env.ENV_ID}}',
    $schema: 'https://framework-1258016615.tcloudbaseapp.com/schema/latest.json',
    framework: {
        name: '{{env.APP_NAME}}',
        plugins: {
            lowcode: {
                use: '@cloudbase/framework-plugin-low-code@0.3.14',
                inputs: {
                    debug: true,
                },
            },
        },
        tags: ['lowcode', 'local'],
    },
};
const DEFAULT_FILENAME = {
    RC: 'cloudbaserc.json',
    INPUT: 'input.json',
    STATUS: 'buildstatus.json',
    ENV: '.env',
    QRCODE: 'qrcode.jpg',
};
var BUILD_STATUE;
(function (BUILD_STATUE) {
    // 构建中
    BUILD_STATUE["BUILDING"] = "building";
    // 构建失败
    BUILD_STATUE["BUILD_FAILED"] = "buildFail";
    // 构建成功
    BUILD_STATUE["BUILD_SUCCESS"] = "success";
    // 安装失败
    BUILD_STATUE["INSTALL_FAILED"] = "fail";
})(BUILD_STATUE || (BUILD_STATUE = {}));
const BUILD_TAG = {
    building: '构建中',
    buildFail: '构建失败',
    success: '构建成功',
    fail: '安装失败',
};
const BUILD_TIME_OUT = 600000;
const tcbService = cloud_api_1.CloudApiService.getInstance({ service: 'tcb' });
const getFileUrl = (path) => (0, url_1.format)({ protocol: 'file:', host: path });
const getDotenvStr = (env) => {
    return Object.keys(env).reduce((content, key) => {
        return `${key}=${env[key]}\n${content}`;
    }, '');
};
class Builder {
    constructor(workspace, info) {
        this._workspace = workspace;
        this._info = info;
    }
    async build(ctx, params) {
        // 0. 初始化
        const { envId, appId, mainAppSerializeData } = params;
        const mainAppData = JSON.parse(mainAppSerializeData);
        const fixedParams = {
            ...params,
            mainAppSerializeData: {
                ...mainAppData,
                dataSources: params.datasources || [],
            },
            datasources: null,
        };
        const buildId = Date.now().toString();
        const env = {
            ENV_ID: envId,
            APP_NAME: `lcap-localbuild-${appId}`,
        };
        // 1. 写入文件，触发构建
        await this._writeFiles(appId, buildId, [
            [DEFAULT_FILENAME.INPUT, JSON.stringify(fixedParams, null, 4)],
            [DEFAULT_FILENAME.ENV, getDotenvStr(env)],
            // rc文件会触发构建, 所以必须最后写入
            [DEFAULT_FILENAME.RC, JSON.stringify(DEFAULT_RC_CONTENT, null, 4)],
        ]);
        // 2. 回包，启动成功
        return {
            buildId,
        };
    }
    async _writeFiles(appId, buildId, contents) {
        const buildPath = (0, path_1.resolve)(this._workspace, appId, buildId);
        (0, fs_1.mkdirSync)(buildPath, { recursive: true });
        contents.forEach((content) => {
            (0, fs_1.writeFileSync)((0, path_1.resolve)(buildPath, content[0]), content[1]);
        });
    }
    async checkStatusInLocal(ctx, params) {
        var _a;
        // 初始化
        const { buildId, appId } = params;
        const buildPath = (0, path_1.resolve)(this._workspace, appId, buildId);
        const statusPath = (0, path_1.resolve)(buildPath, DEFAULT_FILENAME.STATUS);
        const inputsPath = (0, path_1.resolve)(buildPath, DEFAULT_FILENAME.INPUT);
        const isTimeOut = Date.now() - parseInt(buildId, 10) >= BUILD_TIME_OUT;
        // 找不到构建目录，安装失败
        if (!(0, toolbox_1.checkFullAccess)(buildPath)) {
            return {
                status: BUILD_STATUE.INSTALL_FAILED,
            };
        }
        // 找不到status, 构建中
        if (!(0, toolbox_1.checkFullAccess)(statusPath)) {
            return {
                status: isTimeOut ? BUILD_STATUE.INSTALL_FAILED : BUILD_STATUE.BUILDING,
            };
        }
        const status = (_a = getContent(statusPath)) === null || _a === void 0 ? void 0 : _a.status;
        const inputs = getContent(inputsPath);
        if (!status || !inputs) {
            // 构建中, 如果读不到status和inputs，则应该报错
            return {
                status: BUILD_STATUE.INSTALL_FAILED,
            };
        }
        // 根据status回包
        switch (status) {
            case BUILD_STATUE.BUILDING: {
                return {
                    status: isTimeOut ? BUILD_STATUE.INSTALL_FAILED : BUILD_STATUE.BUILDING,
                };
            }
            case BUILD_STATUE.BUILD_FAILED: {
                return { status };
            }
            case BUILD_STATUE.BUILD_SUCCESS: {
                const allDist = await this._getAllDist(params, inputs);
                return {
                    status,
                    ...allDist,
                };
            }
            default: {
                return {
                    status: BUILD_STATUE.INSTALL_FAILED,
                };
            }
        }
    }
    // async checkStatusInCloud(ctx: Context, params:ICheckStatusParams) {
    //   // 初始化
    //   const { buildId, envId } = params
    //   // 通过云API查询状态
    //   const res = await tcbService.request('DescribeCloudBaseProjectVersion', {
    //     EnvId: envId,
    //     CIId: buildId
    //   })
    //   const status = res.Project?.Status
    //   if (!status) {
    //     return {
    //       status: BUILD_STATUE.INSTALL_FAILED
    //     }
    //   }
    //   // 根据status回包
    //   switch(status) {
    //     case BUILD_STATUE.BUILDING:
    //     case BUILD_STATUE.BUILD_FAILED: {
    //       return { status }
    //     }
    //     case BUILD_STATUE.BUILD_SUCCESS: {
    //       const allDist = await this._getAllDist(params)
    //       return { status, ...allDist }
    //     }
    //     default: {
    //       return {
    //         status: BUILD_STATUE.INSTALL_FAILED
    //       }
    //     }
    //   }
    // }
    // 获取构建产物
    async _getAllDist(params, inputs) {
        var _a;
        const allDist = {};
        const { buildId, distType, appId } = params;
        const { envId, deployOptions: { mode }, } = inputs;
        // 获取入口二维码
        if (distType.includes('qrcode')) {
            const qrcodePath = (0, path_1.resolve)(this._workspace, appId, buildId, DEFAULT_FILENAME.QRCODE);
            if ((0, toolbox_1.checkFullAccess)(qrcodePath)) {
                const base64 = (0, fs_1.readFileSync)(qrcodePath, { encoding: 'base64' });
                allDist['qrcode'] = `data:image/jpg;base64,${base64}`;
            }
        }
        // 获取静态网站地址
        if (distType.includes('web')) {
            await (0, toolbox_1.checkAndGetCredential)();
            const hostings = await getHostingInfo(envId);
            const website = (_a = hostings === null || hostings === void 0 ? void 0 : hostings.data) === null || _a === void 0 ? void 0 : _a[0];
            if (website) {
                const rootPath = getWebRootPath(appId, mode);
                allDist['web'] = `https://${(0, path_1.join)(website.cdnDomain, rootPath)}`;
            }
        }
        // 获取代码包
        if (distType.includes('dist')) {
            const distPath = (0, path_1.resolve)(this._workspace, appId, buildId, 'dist');
            const distZipPath = (0, path_1.resolve)(this._workspace, appId, buildId, 'dist.zip');
            if ((0, toolbox_1.checkFullAccess)(distZipPath)) {
                allDist['dist'] = getFileUrl(distZipPath);
            }
            else if ((0, toolbox_1.checkFullAccess)(distPath)) {
                await (0, toolbox_1.zipDir)(distPath, distZipPath);
                allDist['dist'] = getFileUrl(distZipPath);
            }
        }
        return allDist;
    }
    async deployHistory(ctx, params) {
        const { appId, pageIndex = 1, pageSize = 10 } = params;
        const appPath = (0, path_1.resolve)(this._workspace, appId);
        if (!(0, toolbox_1.checkFullAccess)(appPath)) {
            ctx.throw(404, `unknown appId: ${appId}`);
        }
        const historyPaths = (0, fs_1.readdirSync)(appPath)
            .filter((path) => {
            if (!(0, toolbox_1.isDirectorySync)((0, path_1.resolve)(appPath, path))) {
                return false;
            }
            if (!(0, toolbox_1.checkFullAccess)((0, path_1.resolve)(appPath, path, DEFAULT_FILENAME.STATUS))) {
                return false;
            }
            if (!/[0-9]{13}/.test(path)) {
                return false;
            }
            return true;
        })
            .reverse();
        const startIndex = pageSize * (pageIndex - 1);
        const endIndex = pageSize * pageIndex - 1;
        const history = await Promise.all(historyPaths.slice(startIndex, endIndex + 1).map(async (path) => {
            const result = await this.checkStatusInLocal(ctx, {
                appId,
                buildId: path,
                distType: ['qrcode', 'web', 'dist'],
            }).catch((e) => {
                return { status: BUILD_STATUE.BUILDING };
            });
            return {
                appId,
                idx: path,
                ciId: path,
                createdAt: path,
                status: result.status,
                preview: lodash_1.default.pick(result, 'web', 'qrcode', 'dist'),
            };
        }));
        return {
            count: history.length,
            rows: history,
        };
    }
    async deployDetail(ctx, params) {
        const { idx, appId } = params;
        const result = await this.checkStatusInLocal(ctx, {
            buildId: idx,
            distType: ['web', 'qrcode', 'dist'],
            appId,
        });
        return {
            appId,
            idx,
            ciId: idx,
            createdAt: idx,
            status: result.status,
            preview: lodash_1.default.pick(result, 'web', 'qrcode', 'dist'),
        };
    }
}
exports.Builder = Builder;
class Watcher {
    constructor(workspace) {
        this._workspace = workspace;
    }
    start() {
        if (this._watcher) {
            return;
        }
        if (!(0, toolbox_1.checkFullAccess)(this._workspace)) {
            (0, fs_1.mkdirSync)(this._workspace, { recursive: true });
        }
        this._watcher = (0, chokidar_1.watch)(this._workspace, { ignoreInitial: true }).on('all', async (eventName, path) => {
            switch (eventName) {
                case 'add':
                case 'change': {
                    const parse = (0, path_1.parse)(path);
                    if (parse.base === 'cloudbaserc.json' &&
                        // 防止重复构建
                        !(0, toolbox_1.checkFullAccess)((0, path_1.resolve)(parse.dir, DEFAULT_FILENAME.STATUS))) {
                        this._setStatus(parse.dir, BUILD_STATUE.BUILDING);
                        await this._callFramework(parse.dir)
                            .then(() => {
                            this._setStatus(parse.dir, BUILD_STATUE.BUILD_SUCCESS);
                        })
                            .catch(() => {
                            this._setStatus(parse.dir, BUILD_STATUE.BUILD_FAILED);
                        });
                        // this._clean(parse.dir)
                    }
                    break;
                }
                default: {
                    break;
                }
            }
        });
    }
    async _callFramework(projectPath, command = DEFAULT_FRAMEWORK_COMMAND) {
        return new Promise((resolve, reject) => {
            var _a, _b;
            const cmd = (0, child_process_1.exec)(command, {
                cwd: projectPath,
                env: { ...{ CI: 'true' }, ...process.env },
            });
            (_a = cmd.stdout) === null || _a === void 0 ? void 0 : _a.pipe(process.stdout);
            (_b = cmd.stderr) === null || _b === void 0 ? void 0 : _b.pipe(process.stderr);
            cmd.on('exit', (code) => {
                if (code === 0) {
                    resolve(code);
                }
                else {
                    reject(code);
                }
            });
        });
    }
    /**
     * 设置构建状态
     */
    _setStatus(path, status) {
        const statusPath = (0, path_1.resolve)(path, DEFAULT_FILENAME.STATUS);
        const content = JSON.stringify({ status }, null, 4);
        (0, fs_1.writeFileSync)(statusPath, content, { encoding: 'utf8' });
        const buildId = (0, path_1.parse)(path).name;
        const appId = (0, path_1.parse)((0, path_1.resolve)(path, '..')).name;
        console.log(chalk_1.default.green(`\n[微搭低代码] 低码应用${BUILD_TAG[status]} [ appId:${appId}, buildId:${buildId} ]\n`));
    }
    /**
     * 清除源码
     */
    _clean(path) {
        (0, fs_1.unlinkSync)((0, path_1.resolve)(path, DEFAULT_FILENAME.INPUT));
        (0, fs_1.unlinkSync)((0, path_1.resolve)(path, DEFAULT_FILENAME.ENV));
        (0, fs_1.unlinkSync)((0, path_1.resolve)(path, DEFAULT_FILENAME.RC));
    }
    end() {
        var _a;
        (_a = this._watcher) === null || _a === void 0 ? void 0 : _a.close();
    }
}
exports.Watcher = Watcher;
function getWebRootPath(appId, mode) {
    return mode === 'preview' ? `/${appId}/preview/` : `/${appId}/production/`;
}
function getContent(path) {
    try {
        (0, toolbox_1.checkFullAccess)(path, true);
        const rawData = (0, fs_1.readFileSync)(path, 'utf8');
        return JSON.parse(rawData);
    }
    catch (e) {
        return;
    }
}
async function getHostingInfo(envId) {
    const res = await tcbService.request('DescribeStaticStore', {
        EnvId: envId,
    });
    const data = (0, toolbox_1.firstLetterToLowerCase)(res);
    return data;
}
