"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.bootstrap = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const child_process_1 = require("child_process");
const util_1 = require("util");
const lodash_1 = require("lodash");
const chalk_1 = __importDefault(require("chalk"));
const enquirer_1 = require("enquirer");
const obtain_git_repo_1 = require("obtain-git-repo");
const toolbox_1 = require("@cloudbase/toolbox");
const templateChoices = new Map([
    ['default', 'TencentCloudBase/lowcode-comps-template-default'],
    ['typescript', 'TencentCloudBase/lowcode-comps-template-typescript'],
    ['vue', 'TencentCloudBase/lowcode-comps-template-vue']
]);
async function bootstrap(compName, log) {
    // 拼接模板路径
    const compsPath = path_1.default.resolve(process.cwd(), compName);
    if (fs_extra_1.default.pathExistsSync(compsPath)) {
        throw new toolbox_1.CloudBaseError(`当前目录下已存在组件库 ${compName} ！`);
    }
    const { templateName } = await (0, enquirer_1.prompt)({
        type: 'select',
        name: 'templateName',
        message: '请选择组件库模板',
        choices: [...templateChoices.keys()],
    });
    // 下载模板
    await download(compsPath, compName, templateChoices.get(templateName));
    // 安装依赖
    const installed = await installDependencies(compsPath);
    // 用户提示
    log.success('组件库 - 创建成功\n');
    log.info(`👉 执行命令 ${chalk_1.default.bold.cyan(`cd ${compName}`)} 进入文件夹`);
    if (!installed) {
        log.info(`👉 执行命令 ${chalk_1.default.bold.cyan('npm install')} 手动安装依赖`);
    }
    log.info(`👉 执行命令 ${chalk_1.default.bold.cyan('tcb lowcode debug')} 调试组件库`);
    log.info(`👉 执行命令 ${chalk_1.default.bold.cyan('tcb lowcode publish')} 发布组件库`);
}
exports.bootstrap = bootstrap;
async function download(compPath, compName, templateGithubRepo) {
    await (0, toolbox_1.execWithLoading)(async () => {
        await degitClone(templateGithubRepo, compPath);
        // 修改cloudbaserc.json
        await renamePackage(path_1.default.resolve(compPath, 'cloudbaserc.json'), compName);
    }, {
        startTip: '组件库 - 下载模板中',
        successTip: '组件库 - 下载模板成功',
    });
}
async function renamePackage(configPath, name) {
    if (!fs_extra_1.default.existsSync(configPath)) {
        throw new toolbox_1.CloudBaseError(`组件库缺少配置文件: ${configPath}`);
    }
    const rcJson = await fs_extra_1.default.readJSON(configPath);
    const newRcJson = (0, lodash_1.set)(rcJson, 'lowcodeCustomComponents.name', name);
    await fs_extra_1.default.writeJSON(configPath, newRcJson, { spaces: 2 });
}
async function installDependencies(compPath) {
    try {
        await (0, toolbox_1.execWithLoading)(async () => {
            const npmOptions = [
                '--prefer-offline',
                '--no-audit',
                '--progress=false',
                '--registry=https://mirrors.tencent.com/npm/',
                '--legacy-peer-deps',
            ];
            await (0, util_1.promisify)(child_process_1.exec)(['npm install', ...npmOptions].join(' '), {
                cwd: compPath,
                env: process.env,
            });
        }, {
            startTip: '组件库 - 依赖安装中',
            successTip: '组件库 - 依赖安装成功',
            failTip: '组件库 - 依赖安装失败, 请手动安装依赖',
        });
        return true;
    }
    catch (_e) {
        return false;
    }
}
async function degitClone(repo, dest) {
    return new Promise((resolve, reject) => {
        (0, obtain_git_repo_1.download)(repo, dest, { clone: false }, (err) => {
            if (err) {
                reject(err);
            }
            else {
                resolve();
            }
        });
    });
}
