"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.graceBuildComps = exports.GraceBuildService = exports.PUBLIC_PATH_TO_BE_SUBSITITUE = exports.WEB_ACTION_ENTRY_REG = exports.WEB_COMP_ENTRY_REG = exports.WEB_ENTRY_REG = exports.MP_ACTION_ENTRY_REG = exports.MP_COMP_ENTRY_REG = exports.MP_ENTRY_REG = exports.CONFIG_ENTRY_REG = void 0;
/**
 * 重构weapps组件库的构建服务
 */
const lodash_1 = __importDefault(require("lodash"));
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const toolbox_1 = require("@cloudbase/toolbox");
const common_1 = require("../../common");
const webpack_1 = require("../webpack");
exports.CONFIG_ENTRY_REG = /^index\.[jt]{1}s[x]{0,1}$/;
exports.MP_ENTRY_REG = /^index\.json$/;
exports.MP_COMP_ENTRY_REG = /^.+\/index\.wxml$/;
exports.MP_ACTION_ENTRY_REG = /^.+\/index\.[jt]{1}s$/;
exports.WEB_ENTRY_REG = /^index\.[jt]{1}s[x]{0,1}$/;
exports.WEB_COMP_ENTRY_REG = /^.+\/index\.[jt]{1}s[x]{0,1}$/;
exports.WEB_ACTION_ENTRY_REG = /^.+\/index\.[jt]{1}s[x]{0,1}$/;
exports.PUBLIC_PATH_TO_BE_SUBSITITUE = 'SOME_PUBILC_PATH_PLACEHOLDER_TO_SUBSITITUE'.split('').reverse().join('');
class GraceBuildService {
    constructor(info) {
        this.tempPath = '.temp';
        this.outputPath = './.temp/dist';
        this.configEntryPath = './.temp/config.js';
        this.mainEntryPath = './.temp/main.js';
        this.mpMainEntryPath = './.temp/main_mp.js';
        this.lcdsConfigEntryPath = './.temp/lcds.js';
        this.info = info;
        (0, common_1.showCliVersion)();
        (0, common_1.initLowcodeCloudApiEnv)(info);
    }
    async build() {
        // 清理Dist和Temp
        await this.clean();
        // 自动识别
        await this.recognition();
        await (0, toolbox_1.execWithLoading)(async () => {
            // 启动Webpack
            await this.runWebpack();
        }, {
            startTip: '组件库 - 构建中',
            successTip: '组件库 - 构建成功',
        });
    }
    async clean() {
        this.info.logger.info('组件库 - 清空上次构建内容');
        const { context } = this.info;
        // 清理Temp
        await fs_extra_1.default.remove(path_1.default.resolve(context, this.tempPath));
    }
    /**
     * 自动识别
     */
    async recognition() {
        // 配置识别
        await this.configRecognition();
        // 源码识别
        await this.codeRecognitionForWeb();
        await this.codeRecognitionForMp();
    }
    /**
     * 配置识别
     */
    async configRecognition() {
        this.info.logger.info('组件库 - 组件配置识别');
        let configEntry;
        let hasConfigEntry = false;
        const { context } = this.info;
        const config = this.getConfig(this.info);
        const configRoot = lodash_1.default.get(config, 'configRoot');
        const compFiles = lodash_1.default.get(config, 'compFiles');
        const actionFiles = lodash_1.default.get(config, 'actionFiles');
        if (configRoot && !(compFiles && actionFiles)) {
            // 手动指定路径
            hasConfigEntry = true;
            configEntry = `
        const config = () => (
          require('${(0, common_1.pathEscape)(path_1.default.resolve(configRoot, 'index'))}').default
        )
      `;
        }
        else if (compFiles && actionFiles) {
            // 使用约定路径
            configEntry = `
        const config = () => ({
          components: [
            ${compFiles
                .map((file) => `require('${(0, common_1.pathEscape)(path_1.default.resolve(configRoot, 'components', file))}')`)
                .join(',\n')}
          ],
          actions: [
            ${actionFiles
                .map((file) => `require('${(0, common_1.pathEscape)(path_1.default.resolve(configRoot, 'actions', file))}')`)
                .join(',\n')}
          ],
        })
      `;
        }
        // 生成配置
        const finalConfigEntry = `
      ${configEntry}

      // 使用内置的转换函数处理配置
      const transfer = wedaConfigTransfer
      module.exports = transfer(config(), require('./main_mp.js'))

      // 热更新逻辑
      if (module.hot && ${hasConfigEntry}) {
        module.hot.accept('${(0, common_1.pathEscape)(path_1.default.relative(this.tempPath, this.info.configRoot))}', () => {
          const transferConfig = transfer(config())
          window.$$transferConfig = transferConfig
          window.$$hotUpdate ?
            window.$$hotUpdate({...transferConfig, materialName: '${this.info.name}' }) :
            window.location.reload()
        })
      }
    `;
        await fs_extra_1.default.ensureFile(path_1.default.resolve(context, this.configEntryPath));
        await fs_extra_1.default.writeFile(path_1.default.resolve(context, this.configEntryPath), finalConfigEntry);
    }
    /**
     * 配置识别。"configRoot": "./src/configs/" 查找根目录index，找不到，则直接找根目录components、actions
     * @param info cloudbaserc.json中的lowcodeCustomComponents
     * @returns
     */
    getConfig(info) {
        const configRoot = lodash_1.default.get(info, 'configRoot');
        const context = lodash_1.default.get(info, 'context');
        if (!configRoot) {
            throw new toolbox_1.CloudBaseError(`请检查cloudbasetc.json文件中配置的 lowcodeCustomComponents.configRoot 是否填写`);
        }
        if (configRoot && !fs_extra_1.default.existsSync(path_1.default.resolve(context, configRoot))) {
            throw new toolbox_1.CloudBaseError(`请检查cloudbasetc.json文件中配置的 lowcodeCustomComponents.configRoot => ${configRoot} 路径是否存在`);
        }
        const resolveConfigRoot = path_1.default.resolve(context, configRoot);
        // 查找src/configs/index文件是否存在
        if ((0, common_1.findFiles)(exports.CONFIG_ENTRY_REG, resolveConfigRoot).length > 0) {
            return {
                configRoot,
            };
        }
        // 找不到src/configs/index文件，则查找src/configs/components、src/configs/actions下json文件
        const compFiles = (0, common_1.findFiles)(/^.+\.json/, path_1.default.resolve(resolveConfigRoot, 'components'));
        const actionFiles = (0, common_1.findFiles)(/^.+\.json/, path_1.default.resolve(resolveConfigRoot, 'actions'));
        return {
            configRoot,
            compFiles,
            actionFiles,
        };
    }
    /**
     * 源码识别(web)
     */
    async codeRecognitionForWeb() {
        this.info.logger.info('组件库 - 组件源码识别(web)');
        let mainEntry;
        const { context } = this.info;
        const config = this.getPlatformConfig('web', this.info);
        const platformEntry = lodash_1.default.get(config, 'platformEntry');
        const compFiles = lodash_1.default.get(config, 'compFiles');
        const actionFiles = lodash_1.default.get(config, 'actionFiles');
        if (platformEntry && !(compFiles && actionFiles)) {
            // 手动指定路径
            mainEntry = `
          const main = require('${(0, common_1.pathEscape)(path_1.default.resolve(platformEntry, 'index'))}').default

          module.exports = {
            schemaVersion: '${common_1.COMPONENTS_CONFIG_SCHEMA_VERSION}',
            components: main.components || {},
            actions: main.actions || {},
            plugins: []
          }

          if (
            module.hot &&
            document.getElementsByClassName('is-editor-type').length > 0
          ) {
            console.log('[WEDA]','Hmr Init')
            module.hot.accept('${(0, common_1.pathEscape)(path_1.default.relative(this.tempPath, platformEntry))}', () => {
              console.log('[WEDA]','Hmr Update')
              window.top[1]?.$$refreshSimulator ?
                window.top[1]?.$$refreshSimulator() :
                window.top[1]?.location?.reload()
            })
          }
        `;
        }
        else if (compFiles && actionFiles) {
            // 使用约定路径
            mainEntry = `
          module.exports = {
            schemaVersion: '${common_1.COMPONENTS_CONFIG_SCHEMA_VERSION}',
            components: {
              ${compFiles
                .map((file) => {
                const name = path_1.default.parse(file).dir;
                const resolvePath = path_1.default.resolve(platformEntry, 'components', file);
                return `${name}: require('${(0, common_1.pathEscape)(resolvePath)}').default`;
            })
                .join(',\n')}
            },
            plugins: [],
            actions: {
              ${actionFiles
                .map((file) => {
                const name = path_1.default.parse(file).dir;
                const resolvePath = path_1.default.resolve(platformEntry, 'actions', file);
                return `${name}: require('${(0, common_1.pathEscape)(resolvePath)}').default`;
            })
                .join(',\n')}
            },
          }
        `;
        }
        else {
            // 用户没有实现WEB组件
            mainEntry = `
        module.exports = {
          schemaVersion: '${common_1.COMPONENTS_CONFIG_SCHEMA_VERSION}',
          components: {},
          plugins: [],
          actions: {},
        }
      `;
        }
        const resolvePath = path_1.default.resolve(context, this.mainEntryPath);
        await fs_extra_1.default.ensureFile(resolvePath);
        await fs_extra_1.default.writeFile(resolvePath, mainEntry);
    }
    /**
     * platform配置识别
        "platformRoot": {
          "web": "./src/xxx"
        }
        查找根目录index，找不到，则直接找根目录components、actions
      * @param platform 平台：web/mp
      * @param info cloudbaserc.json中的lowcodeCustomComponents
      * @returns
    */
    getPlatformConfig(platform, info) {
        let platformConfig = lodash_1.default.get(info, `platformRoot.${platform}`);
        if (!platformConfig) {
            throw new toolbox_1.CloudBaseError(`请检查cloudbasetc.json文件中配置的 lowcodeCustomComponents.platformRoot.${platform} 是否填写`);
        }
        const platformEntry = this.getPlatformEntry(platformConfig);
        let entryReg = exports.WEB_ENTRY_REG;
        let compReg = exports.WEB_COMP_ENTRY_REG;
        let actionReg = exports.WEB_ACTION_ENTRY_REG;
        const context = lodash_1.default.get(info, 'context');
        const resolvePlatformRoot = path_1.default.resolve(context, platformEntry); // 入口绝对路径
        if (platform === 'mp') {
            entryReg = exports.MP_ENTRY_REG;
            compReg = exports.MP_COMP_ENTRY_REG;
            actionReg = exports.MP_ACTION_ENTRY_REG;
        }
        if (platformEntry && !fs_extra_1.default.existsSync(path_1.default.resolve(context, platformEntry))) {
            throw new toolbox_1.CloudBaseError(`请检查cloudbasetc.json文件中配置的 lowcodeCustomComponents.platformRoot.${platform}.entry => ${platformEntry} 路径是否存在`);
        }
        if ((0, common_1.findFiles)(entryReg, resolvePlatformRoot).length > 0) {
            return {
                platformEntry,
            };
        }
        // 找不到src/[platform]/index文件，则查找src/[platform]/components、src/[platform]/actions下index文件
        const compFiles = (0, common_1.findFiles)(compReg, path_1.default.resolve(platformEntry, 'components'), true);
        const actionFiles = (0, common_1.findFiles)(actionReg, path_1.default.resolve(platformEntry, 'actions'), true);
        return {
            platformEntry,
            compFiles,
            actionFiles,
        };
    }
    getPlatformEntry(platformConfig) {
        if (!this.isVersion3(platformConfig)) {
            return lodash_1.default.get(platformConfig, 'entry');
        }
        return platformConfig;
    }
    /**
     * 源码识别(mp)
     */
    async codeRecognitionForMp() {
        this.info.logger.info('组件库 - 组件源码识别(mp)');
        let mainEntry;
        const { context } = this.info;
        const config = this.getPlatformConfig('mp', this.info);
        const platformEntry = lodash_1.default.get(config, 'platformEntry');
        const compFiles = lodash_1.default.get(config, 'compFiles');
        const actionFiles = lodash_1.default.get(config, 'actionFiles');
        if (platformEntry && !(compFiles && actionFiles)) {
            // 手动指定路径
            mainEntry = `
        const main = require('${(0, common_1.pathEscape)(path_1.default.resolve(platformEntry, 'index.json'))}')
        module.exports = main
      `;
        }
        else if (compFiles && actionFiles) {
            // 使用约定路径 这里有问题，mp/components/下没有index，小程序
            mainEntry = `
        module.exports = {
          components: {
            ${compFiles
                .map((file) => {
                const name = path_1.default.parse(file).dir;
                return `${name}: 'components/${name}/index'`;
            })
                .join(',\n')}
          },
          plugins: [],
          actions: {
            ${actionFiles
                .map((file) => {
                const name = path_1.default.parse(file).dir;
                return `${name}: 'actions/${name}/index'`;
            })
                .join(',\n')}
          },
        }
      `;
        }
        else {
            // 用户没有实现MP组件
            mainEntry = `
        module.exports = {
          components: {},
          plugins: [],
          actions: {},
        }
      `;
        }
        const resolvePath = path_1.default.resolve(context, this.mpMainEntryPath);
        await fs_extra_1.default.ensureFile(resolvePath);
        await fs_extra_1.default.writeFile(resolvePath, mainEntry);
    }
    async runWebpack() {
        const { name, context, splitChunks = false, sourceMap } = this.info;
        const isTs = this.isTs(this.info);
        // 打包 main 和 control 文件
        const mainService = new webpack_1.WebpackService({
            name,
            context,
            entry: {
                main: this.mainEntryPath,
            },
            output: this.outputPath,
            mode: 'production',
            isTs,
            splitChunks,
            publicPath: 'auto',
            sourceMap,
        });
        // 打包lcds 文件
        const controlService = new webpack_1.WebpackService({
            name,
            context,
            entry: {
                control: this.configEntryPath,
            },
            output: this.outputPath,
            mode: 'production',
            isTs,
            splitChunks: false,
            publicPath: 'auto',
            sourceMap: false,
        });
        await Promise.all([mainService.webpack(), controlService.webpack()]);
        // 打包lcds 文件
        const lcdsService = new webpack_1.WebpackService({
            name,
            context,
            entry: {
                lcds: this.configEntryPath,
            },
            output: this.outputPath,
            mode: 'production',
            libraryTarget: 'this',
            isTs,
            splitChunks: false,
            publicPath: 'auto',
            sourceMap: false,
        });
        await lcdsService.webpack();
    }
    isTs(info) {
        const { context } = info;
        return !!((0, common_1.findFiles)(/^tsconfig\.json$/, context).length &&
            (0, common_1.findFiles)(/^(.+\/){0,1}index\.ts[x]{0,1}$/, path_1.default.resolve(context, this.getPlatformEntry(lodash_1.default.get(info.platformRoot, 'web'))), true).length);
    }
    parseWeda(platform, info) {
        const platformEntry = this.getPlatformEntry(lodash_1.default.get(info, `platformRoot.${platform}`));
        const context = lodash_1.default.get(info, 'context');
        const configRoot = lodash_1.default.get(info, 'configRoot');
        const configEntryFile = lodash_1.default.get((0, common_1.findFiles)(exports.CONFIG_ENTRY_REG, path_1.default.resolve(context, configRoot)), '0', '');
        const platformEntryFile = lodash_1.default.get((0, common_1.findFiles)(platform === 'web' ? exports.WEB_ENTRY_REG : exports.MP_ENTRY_REG, path_1.default.resolve(context, platformEntry)), '0', '');
        const platformConfig = {
            entry: `${platformEntry}/${platformEntryFile}`,
        };
        if (platform === 'web') {
            const platformCompFile = lodash_1.default.get((0, common_1.findFiles)(exports.WEB_ENTRY_REG, path_1.default.resolve(context, platformEntry, 'components'), true), '0', '');
            const platformActionFile = lodash_1.default.get((0, common_1.findFiles)(exports.WEB_ENTRY_REG, path_1.default.resolve(context, platformEntry, 'actions'), true), '0', '');
            platformConfig.components = `${platformEntry}/components/${platformCompFile}`;
            platformConfig.actions = `${platformEntry}/actions/${platformActionFile}`;
        }
        return {
            config: `${configRoot}/${configEntryFile}`,
            platform: {
                [platform]: platformConfig,
            },
        };
    }
    /**
     * 生成package.json
     * @description
     * 旧的：web下package.json添加lowcode字段。
     * {
        "main": "./src/web/index.js",
        "lowcode": "./src/web/index.js",
       }
     * 新的：web和mp下package.json添加weda字段，main、lowcode保留。字段有区别。如下举例，mp没有components、actions
     * web：
     * {
          "main": "./src/web/index.js",
          "lowcode": "./src/web/index.js",
          "weda": {
          "config": "./src/configs/index.js",
          "platform": {
            "web": {
              "entry": "./src/web/index.js",
              "components": "./src/web/components/index.js",
              "actions": "./src/web/actions/index.js"
            }
          }
        }
        mp:
        {
          "main": "./src/web/index.js",
          "lowcode": "./src/web/index.js",
          "weda": {
          "config": "./src/configs/index.js",
          "platform": {
            "web": {
              "entry": "./src/mp/index.js"
            }
          }
        }
     *
     */
    async generatePkg(platform, platformInfo, codeRoot) {
        var _a;
        const isVersion3 = this.isVersion3(lodash_1.default.get(this.info, `platformRoot.${platform}`));
        if (isVersion3 && platform === 'mp') {
            return;
        }
        const { context } = this.info; // 项目根路径
        const { sourcePath } = platformInfo;
        const packageSrcPath = path_1.default.resolve(context, 'package.json');
        const packageDistPath = path_1.default.resolve(context, sourcePath, 'package.json');
        if (!fs_extra_1.default.existsSync(packageSrcPath)) {
            return;
        }
        const info = await fs_extra_1.default.readJSON(packageSrcPath);
        const entry = (_a = (0, common_1.findFiles)(platform === 'web' ? exports.WEB_ENTRY_REG : exports.MP_ENTRY_REG, path_1.default.resolve(context, sourcePath, platform === 'web' ? codeRoot : 'src'))) === null || _a === void 0 ? void 0 : _a[0];
        const wedaConfig = this.parseWeda(platform, this.info);
        const weda = !!Object.keys(wedaConfig).length ? { weda: wedaConfig } : {};
        const fixedInfo = {
            ...(platform === 'web' ? info : this.generateMpPkg(info)),
            ...(isVersion3 && entry ? { main: `${codeRoot}/${entry}`, lowcode: `${codeRoot}/${entry}` } : {}),
            ...(!isVersion3 && weda),
        };
        await fs_extra_1.default.ensureFile(packageDistPath);
        await fs_extra_1.default.writeJSON(packageDistPath, fixedInfo, { spaces: 2 });
    }
    generateMpPkg(info) {
        if (lodash_1.default.isObject(info)) {
            const { dependencies, devDependencies, peerDependencies, ...filteredInfo } = info;
            return filteredInfo;
        }
        return info;
    }
    isVersion3(platformEntry) {
        return lodash_1.default.isString(platformEntry);
    }
}
exports.GraceBuildService = GraceBuildService;
async function graceBuildComps(info = {
    name: 'demo',
    configRoot: './configs',
    platformRoot: {
        web: {
            entry: './src/web',
            components: './src/web/components',
            actions: './src/web/actions',
        },
        mp: {
            entry: './src/mp',
        },
    },
    context: process.cwd(),
    command: 'build',
    splitChunks: false,
    sourceMap: false,
}) {
    const service = new GraceBuildService(info);
    await service.build();
}
exports.graceBuildComps = graceBuildComps;
